@@ -20,9 +20,13 @@
  */
 #include "internal.h"
 
+#include <fcntl.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
+#include <sys/stat.h>
+#include <sys/types.h>
+#include <sys/un.h>
 
 #include <osmocom/core/linuxlist.h>
 #include <osmocom/core/talloc.h>
@@ -37,15 +41,16 @@
 #include <osmocom/vty/telnet_interface.h>
 
 #include <osmocom/abis/e1_input.h>
+#include <osmocom/abis/ipa.h>
 
 /* CONFIG */
 
-#define E1_DRIVER_NAMES		"(misdn|misdn_lapd|dahdi|ipa|unixsocket)"
+#define E1_DRIVER_NAMES		"(misdn|misdn_lapd|dahdi|e1d|ipa|unixsocket)"
 #define E1_DRIVER_HELP		"mISDN supported E1 Card (kernel LAPD)\n" \
 				"mISDN supported E1 Card (userspace LAPD)\n" \
 				"DAHDI supported E1/T1/J1 Card\n" \
+				"osmo-e1d supported E1 interface\n" \
 				"IPA TCP/IP input\n" \
-				"HSL TCP/IP input\n" \
 				"Unix socket input\n"
 
 #define E1_LINE_HELP		"Configure E1/T1/J1 Line\n" "Line Number\n"
@@ -98,6 +103,14 @@ DEFUN(cfg_e1line_socket, cfg_e1_line_socket_cmd,
 {
 	struct e1inp_line *line;
 	int e1_nr = atoi(argv[0]);
+	struct sockaddr_un sun;
+
+	/* Don't exceed the maximum unix socket path length, including a NUL byte. See the unix(7) man page.*/
+	if (strlen(argv[1]) > sizeof(sun.sun_path) - 1) {
+		vty_out(vty, "%% Socket path length exceeds %zd bytes: '%s'%s",
+			sizeof(sun.sun_path) - 1, argv[1], VTY_NEWLINE);
+		return CMD_WARNING;
+	}
 
 	line = e1inp_line_find(e1_nr);
 	if (!line) {
@@ -160,6 +173,47 @@ DEFUN(cfg_e1line_no_keepalive, cfg_e1_line_no_keepalive_cmd,
 	return set_keepalive_params(vty, atoi(argv[0]), 0, 0, 0);
 }
 
+#define IPA_KEEPALIVE_HELP "Enable IPA PING/PONG keep-alive\n"
+static int set_ipa_keepalive_params(struct vty *vty, int e1_nr, int interval, int wait_for_resp)
+{
+	struct e1inp_line *line = e1inp_line_find(e1_nr);
+
+	if (!line) {
+		vty_out(vty, "%% Line %d doesn't exist%s", e1_nr, VTY_NEWLINE);
+		return CMD_WARNING;
+	}
+
+	if (strcmp(line->driver->name, "ipa") != 0) {
+		vty_out(vty, "%% Line %d doesn't use the ipa driver%s", e1_nr, VTY_NEWLINE);
+		return CMD_WARNING;
+	}
+
+	TALLOC_FREE(line->ipa_kap);
+	if (interval) {
+		line->ipa_kap = talloc_zero(line, struct ipa_keepalive_params);
+		line->ipa_kap->wait_for_resp = wait_for_resp;
+		line->ipa_kap->interval = interval;
+	}
+
+	return CMD_SUCCESS;
+}
+
+DEFUN(cfg_e1line_ipa_keepalive, cfg_e1_line_ipa_keepalive_cmd,
+	"e1_line <0-255> ipa-keepalive <1-300> <1-300>",
+	E1_LINE_HELP IPA_KEEPALIVE_HELP
+	"Idle interval in seconds before probes are sent\n"
+	"Time to wait for PONG response\n")
+{
+	return set_ipa_keepalive_params(vty, atoi(argv[0]), atoi(argv[1]), atoi(argv[2]));
+}
+
+DEFUN(cfg_e1line_no_ipa_keepalive, cfg_e1_line_no_ipa_keepalive_cmd,
+	"no e1_line <0-255> ipa-keepalive",
+	NO_STR E1_LINE_HELP IPA_KEEPALIVE_HELP)
+{
+	return set_ipa_keepalive_params(vty, atoi(argv[0]), 0, 0);
+}
+
 DEFUN(cfg_e1line_name, cfg_e1_line_name_cmd,
 	"e1_line <0-255> name .LINE",
 	E1_LINE_HELP "Set name for this line\n" "Human readable name\n")
@@ -181,6 +235,32 @@ DEFUN(cfg_e1line_name, cfg_e1_line_name_cmd,
 	return CMD_SUCCESS;
 }
 
+DEFUN(cfg_e1_pcap, cfg_e1_pcap_cmd,
+	"pcap .FILE",
+	"Setup a pcap recording of all E1 traffic\n"
+	"Filename to save the packets to\n")
+{
+	int fd;
+
+	fd = open(argv[0], O_WRONLY | O_CREAT | O_TRUNC, 0660);
+	if (fd < 0) {
+		vty_out(vty, "Failed to setup E1 pcap recording to %s.%s", argv[0], VTY_NEWLINE);
+		return CMD_WARNING;
+	}
+
+	e1_set_pcap_fd(fd);
+
+	return CMD_SUCCESS;
+}
+
+DEFUN(cfg_e1_no_pcap, cfg_e1_no_pcap_cmd,
+	"no pcap",
+	NO_STR "Disable pcap recording of all E1 traffic\n")
+{
+	e1_set_pcap_fd(-1);
+	return CMD_SUCCESS;
+}
+
 DEFUN(cfg_e1inp, cfg_e1inp_cmd,
 	"e1_input",
 	"Configure E1/T1/J1 TDM input\n")
@@ -233,7 +313,10 @@ static int e1inp_config_write(struct vty *vty)
 				line->keepalive_num_probes,
 				line->keepalive_probe_interval,
 				VTY_NEWLINE);
-
+		if (line->ipa_kap)
+			vty_out(vty, " e1_line %u ipa-keepalive %d %d%s", line->num,
+				line->ipa_kap->interval, line->ipa_kap->wait_for_resp,
+				VTY_NEWLINE);
 	}
 
 	const char *ipa_bind = e1inp_ipa_get_bind_addr();
@@ -382,6 +465,9 @@ int e1inp_vty_init(void)
 	install_element(CONFIG_NODE, &cfg_e1inp_cmd);
 	install_node(&e1inp_node, e1inp_config_write);
 
+	install_element(L_E1INP_NODE, &cfg_e1_pcap_cmd);
+	install_element(L_E1INP_NODE, &cfg_e1_no_pcap_cmd);
+
 	install_element(L_E1INP_NODE, &cfg_e1_line_driver_cmd);
 	install_element(L_E1INP_NODE, &cfg_e1_line_port_cmd);
 	install_element(L_E1INP_NODE, &cfg_e1_line_socket_cmd);
@@ -389,6 +475,8 @@ int e1inp_vty_init(void)
 	install_element(L_E1INP_NODE, &cfg_e1_line_keepalive_cmd);
 	install_element(L_E1INP_NODE, &cfg_e1_line_keepalive_params_cmd);
 	install_element(L_E1INP_NODE, &cfg_e1_line_no_keepalive_cmd);
+	install_element(L_E1INP_NODE, &cfg_e1_line_ipa_keepalive_cmd);
+	install_element(L_E1INP_NODE, &cfg_e1_line_no_ipa_keepalive_cmd);
 
 	install_element(L_E1INP_NODE, &cfg_ipa_bind_cmd);
 
