@@ -1,6 +1,6 @@
 /* OpenBSC Abis input driver for DAHDI */
 
-/* (C) 2008-2011 by Harald Welte <laforge@gnumonks.org>
+/* (C) 2008-2019 by Harald Welte <laforge@gnumonks.org>
  * (C) 2009 by Holger Hans Peter Freyther <zecke@selfish.org>
  * (C) 2010 by Digium and Matthew Fredrickson <creslin@digium.com>
  *
@@ -25,8 +25,7 @@
  */
 
 #include "config.h"
-
-#ifdef HAVE_DAHDI_USER_H
+#include "internal.h"
 
 #include <stdio.h>
 #include <unistd.h>
@@ -136,7 +135,7 @@ static void handle_dahdi_exception(struct e1inp_ts *ts)
 	if (rc < 0)
 		return;
 
-	LOGP(DLMI, LOGL_NOTICE, "Line %u(%s) / TS %u DAHDI EVENT %s\n",
+	LOGPITS(ts, DLMI, LOGL_NOTICE, "Line %u(%s) / TS %u DAHDI EVENT %s\n",
 		ts->line->num, ts->line->name, ts->num,
 		get_value_string(dahdi_evt_names, evt));
 
@@ -183,11 +182,13 @@ static int handle_ts1_read(struct osmo_fd *bfd)
 	if (ret == -1)
 		handle_dahdi_exception(e1i_ts);
 	else if (ret < 0) {
-		perror("read ");
+		LOGPITS(e1i_ts, DLMI, LOGL_ERROR, "%s read failed %d (%s)\n", __func__, ret, strerror(errno));
+		return ret;
 	}
 	msgb_put(msg, ret - 2);
 	if (ret <= 3) {
-		perror("read ");
+		LOGPITS(e1i_ts, DLMI, LOGL_ERROR, "%s read failed %d (%s)\n", __func__, ret, strerror(errno));
+		return ret;
 	}
 
 	return e1inp_rx_ts_lapd(e1i_ts, msg);
@@ -199,7 +200,7 @@ static int ts_want_write(struct e1inp_ts *e1i_ts)
 	 * writeset, since it doesn't support poll() based
 	 * write flow control */
 	if (e1i_ts->type == E1INP_TS_TYPE_TRAU) {
-		LOGP(DLINP, LOGL_DEBUG, "Trying to write TRAU ts\n");
+		LOGPITS(e1i_ts, DLINP, LOGL_DEBUG, "Trying to write TRAU ts\n");
 		return 0;
 	}
 
@@ -224,12 +225,21 @@ static void dahdi_write_msg(struct msgb *msg, void *cbdata)
 	struct e1inp_ts *e1i_ts = &line->ts[ts_nr-1];
 	int ret;
 
-	ret = write(bfd->fd, msg->data, msg->len + 2);
+	if (msgb_tailroom(msg) >= 2) {
+		/* two bytes of space for the FCS added by DAHDI in the kernel */
+		msgb_put(msg, 2);
+		ret = write(bfd->fd, msg->data, msg->len);
+	} else {
+		/* work-around for code that sends us messages with no tailroom (OS#4644) */
+		uint8_t buf[msg->len + 2];
+		memcpy(buf, msg->data, msg->len);
+		ret = write(bfd->fd, buf, sizeof(buf));
+	}
 	msgb_free(msg);
 	if (ret == -1)
 		handle_dahdi_exception(e1i_ts);
 	else if (ret < 0)
-		LOGP(DLMI, LOGL_NOTICE, "%s write failed %d\n", __func__, ret);
+		LOGPITS(e1i_ts, DLMI, LOGL_NOTICE, "%s write failed %d\n", __func__, ret);
 }
 
 static int handle_ts1_write(struct osmo_fd *bfd)
@@ -249,7 +259,7 @@ static int handle_ts1_write(struct osmo_fd *bfd)
 		return 0;
 	}
 
-	DEBUGP(DLMI, "TX: %s\n", osmo_hexdump(msg->data, msg->len));
+	LOGPITS(e1i_ts, DLMI, LOGL_ERROR, "TX: %s\n", osmo_hexdump(msg->data, msg->len));
 	lapd_transmit(e1i_ts->lapd, sign_link->tei,
 			sign_link->sapi, msg);
 
@@ -273,12 +283,21 @@ static void handle_hdlc_write(struct osmo_fd *bfd)
 	if (!msg)
 		return;
 
-	ret = write(bfd->fd, msg->data, msg->len + 2);
+	if (msgb_tailroom(msg) >= 2) {
+		/* two bytes of space for the FCS added by DAHDI in the kernel */
+		msgb_put(msg, 2);
+		ret = write(bfd->fd, msg->data, msg->len);
+	} else {
+		/* work-around for code that sends us messages with no tailroom (OS#4644) */
+		uint8_t buf[msg->len + 2];
+		memcpy(buf, msg->data, msg->len);
+		ret = write(bfd->fd, buf, sizeof(buf));
+	}
 	msgb_free(msg);
 	if (ret == -1)
 		handle_dahdi_exception(e1i_ts);
 	else if (ret < 0)
-		LOGP(DLMI, LOGL_NOTICE, "%s write failed %d\n", __func__, ret);
+		LOGPITS(e1i_ts, DLMI, LOGL_NOTICE, "%s write failed %d\n", __func__, ret);
 }
 
 static int handle_hdlc_read(struct osmo_fd *bfd)
@@ -296,11 +315,13 @@ static int handle_hdlc_read(struct osmo_fd *bfd)
 	if (ret == -1)
 		handle_dahdi_exception(e1i_ts);
 	else if (ret < 0) {
-		perror("read ");
+		LOGPITS(e1i_ts, DLMI, LOGL_ERROR, "%s read failed %d (%s)\n", __func__, ret, strerror(errno));
+		return ret;
 	}
 	msgb_put(msg, ret - 2);
 	if (ret <= 3) {
-		perror("read ");
+		LOGPITS(e1i_ts, DLMI, LOGL_ERROR, "%s read failed %d (%s)\n", __func__, ret, strerror(errno));
+		return ret;
 	}
 
 	return e1inp_rx_ts(e1i_ts, msg, 0, 0);
@@ -308,34 +329,6 @@ static int handle_hdlc_read(struct osmo_fd *bfd)
 
 static int invertbits = 1;
 
-static uint8_t flip_table[256];
-
-static void init_flip_bits(void)
-{
-        int i,k;
-
-        for (i = 0 ; i < 256 ; i++) {
-                uint8_t sample = 0 ;
-                for (k = 0; k<8; k++) {
-                        if ( i & 1 << k ) sample |= 0x80 >>  k;
-                }
-                flip_table[i] = sample;
-        }
-}
-
-static uint8_t * flip_buf_bits ( uint8_t * buf , int len)
-{
-        int i;
-        uint8_t * start = buf;
-
-        for (i = 0 ; i < len; i++) {
-                buf[i] = flip_table[(uint8_t)buf[i]];
-        }
-
-        return start;
-}
-
-#define D_BCHAN_TX_GRAN 160
 /* write to a B channel TS */
 static int handle_tsX_write(struct osmo_fd *bfd)
 {
@@ -349,21 +342,19 @@ static int handle_tsX_write(struct osmo_fd *bfd)
 	ret = subchan_mux_out(mx, tx_buf, D_BCHAN_TX_GRAN);
 
 	if (ret != D_BCHAN_TX_GRAN) {
-		LOGP(DLINP, LOGL_DEBUG, "Huh, got ret of %d\n", ret);
+		LOGPITS(e1i_ts, DLINP, LOGL_DEBUG, "Huh, got ret of %d\n", ret);
 		if (ret < 0)
 			return ret;
 	}
 
-	DEBUGP(DLMIB, "BCHAN TX: %s\n",
-		osmo_hexdump(tx_buf, D_BCHAN_TX_GRAN));
+	LOGPITS(e1i_ts, DLMIB, LOGL_DEBUG, "BCHAN TX: %s\n", osmo_hexdump(tx_buf, D_BCHAN_TX_GRAN));
 
-	if (invertbits) {
-		flip_buf_bits(tx_buf, ret);
-	}
+	if (invertbits)
+		osmo_revbytebits_buf(tx_buf, ret);
 
 	ret = write(bfd->fd, tx_buf, ret);
 	if (ret < D_BCHAN_TX_GRAN)
-		LOGP(DLINP, LOGL_DEBUG, "send returns %d instead of %d\n",
+		LOGPITS(e1i_ts, DLINP, LOGL_DEBUG, "send returns %d instead of %d\n",
 			ret, D_BCHAN_TX_GRAN);
 
 	return ret;
@@ -384,20 +375,17 @@ static int handle_tsX_read(struct osmo_fd *bfd)
 
 	ret = read(bfd->fd, msg->data, D_TSX_ALLOC_SIZE);
 	if (ret < 0 || ret != D_TSX_ALLOC_SIZE) {
-		LOGP(DLINP, LOGL_DEBUG, "read error  %d %s\n",
-			ret, strerror(errno));
+		LOGPITS(e1i_ts, DLINP, LOGL_DEBUG, "read error  %d %s\n", ret, strerror(errno));
 		return ret;
 	}
 
-	if (invertbits) {
-		flip_buf_bits(msg->data, ret);
-	}
+	if (invertbits)
+		osmo_revbytebits_buf(msg->data, ret);
 
 	msgb_put(msg, ret);
 
 	msg->l2h = msg->data;
-	DEBUGP(DLMIB, "BCHAN RX: %s\n",
-		osmo_hexdump(msgb_l2(msg), ret));
+	LOGPITS(e1i_ts, DLMIB, LOGL_DEBUG, "BCHAN RX: %s\n", osmo_hexdump(msgb_l2(msg), ret));
 	ret = e1inp_rx_ts(e1i_ts, msg, 0, 0);
 	/* physical layer indicates that data has been sent,
 	 * we thus can send some more data */
@@ -424,21 +412,18 @@ static int handle_ts_raw_write(struct osmo_fd *bfd)
 		/* This might lead to a transmit underrun, as we call tx
 		 * from the rx path, as there's no select/poll on dahdi
 		 * */
-		LOGP(DLINP, LOGL_NOTICE, "unexpected msg->len = %u, "
+		LOGPITS(e1i_ts, DLINP, LOGL_NOTICE, "unexpected msg->len = %u, "
 		     "expected %u\n", msg->len, D_BCHAN_TX_GRAN);
 	}
 
-	DEBUGP(DLMIB, "RAW CHAN TX: %s\n",
-		osmo_hexdump(msg->data, msg->len));
+	LOGPITS(e1i_ts, DLMIB, LOGL_DEBUG, "RAW CHAN TX: %s\n", osmo_hexdump(msg->data, msg->len));
 
-	if (0/*invertbits*/) {
-		flip_buf_bits(msg->data, msg->len);
-	}
+	if (0/*invertbits*/)
+		osmo_revbytebits_buf(msg->data, msg->len);
 
 	ret = write(bfd->fd, msg->data, msg->len);
 	if (ret < msg->len)
-		LOGP(DLINP, LOGL_DEBUG, "send returns %d instead of %d\n",
-			ret, msg->len);
+		LOGPITS(e1i_ts, DLINP, LOGL_DEBUG, "send returns %d instead of %d\n", ret, msg->len);
 	msgb_free(msg);
 
 	return ret;
@@ -457,20 +442,17 @@ static int handle_ts_raw_read(struct osmo_fd *bfd)
 
 	ret = read(bfd->fd, msg->data, D_TSX_ALLOC_SIZE);
 	if (ret < 0 || ret != D_TSX_ALLOC_SIZE) {
-		LOGP(DLINP, LOGL_DEBUG, "read error  %d %s\n",
-			ret, strerror(errno));
+		LOGPITS(e1i_ts, DLINP, LOGL_DEBUG, "read error  %d %s\n", ret, strerror(errno));
 		return ret;
 	}
 
-	if (0/*invertbits*/) {
-		flip_buf_bits(msg->data, ret);
-	}
+	if (0/*invertbits*/)
+		osmo_revbytebits_buf(msg->data, ret);
 
 	msgb_put(msg, ret);
 
 	msg->l2h = msg->data;
-	DEBUGP(DLMIB, "RAW CHAN RX: %s\n",
-		osmo_hexdump(msgb_l2(msg), ret));
+	LOGPITS(e1i_ts, DLMIB, LOGL_DEBUG, "RAW CHAN RX: %s\n", osmo_hexdump(msgb_l2(msg), ret));
 	ret = e1inp_rx_ts(e1i_ts, msg, 0, 0);
 	/* physical layer indicates that data has been sent,
 	 * we thus can send some more data */
@@ -528,8 +510,7 @@ static int dahdi_fd_cb(struct osmo_fd *bfd, unsigned int what)
 		 * write flow control */
 		break;
 	default:
-		LOGP(DLINP, LOGL_NOTICE,
-			"unknown E1 TS type %u\n", e1i_ts->type);
+		LOGPITS(e1i_ts, DLINP, LOGL_NOTICE, "unknown E1 TS type %u\n", e1i_ts->type);
 		break;
 	}
 
@@ -603,6 +584,33 @@ int dahdi_set_bufinfo(int fd, int as_sigchan)
 	return 0;
 }
 
+static int dahdi_open_slot(int dahdi_chan_nr)
+{
+	int rc, fd;
+#ifndef DAHDI_SPECIFY
+	char openstr[128];
+	snprintf(openstr, sizeof(openstr), "/dev/dahdi/%d", dev_nr);
+#else
+	const char *openstr = "/dev/dahdi/channel";
+#endif
+	rc = open(openstr, O_RDWR | O_NONBLOCK);
+	if (rc < 0) {
+		LOGP(DLINP, LOGL_ERROR, "DAHDI: could not open %s %s\n", openstr, strerror(errno));
+		return -EIO;
+	}
+	fd = rc;
+#ifdef DAHDI_SPECIFY
+	rc = ioctl(fd, DAHDI_SPECIFY, &dahdi_chan_nr);
+	if (rc < 0) {
+		close(fd);
+		LOGP(DLINP, LOGL_ERROR, "DAHDI: could not DAHDI_SPECIFY %d: %s\n",
+		     dahdi_chan_nr, strerror(errno));
+		return -EIO;
+	}
+#endif
+	return fd;
+}
+
 static int dahdi_e1_setup(struct e1inp_line *line)
 {
 	struct span_cfg *scfg;
@@ -612,9 +620,8 @@ static int dahdi_e1_setup(struct e1inp_line *line)
 
 	scfg = span_cfgs[line->port_nr];
 	if (!scfg) {
-		LOGP(DLMI, LOGL_ERROR, "Line %u(%s): DAHDI Port %u (Span %u) "
-			"doesn't exist\n", line->num, line->name, line->port_nr,
-			line->port_nr+1);
+		LOGPIL(line, DLMI, LOGL_ERROR, "Line %u(%s): DAHDI Port %u (Span %u) doesn't exist\n",
+			line->num, line->name, line->port_nr, line->port_nr+1);
 		return -EIO;
 	}
 
@@ -623,7 +630,6 @@ static int dahdi_e1_setup(struct e1inp_line *line)
 	/* TS0 is CRC4, don't need any fd for it */
 	for (ts = 1; ts <= scfg->chan_num; ts++) {
 		unsigned int idx = ts-1;
-		char openstr[128];
 		struct e1inp_ts *e1i_ts = &line->ts[idx];
 		struct osmo_fd *bfd = &e1i_ts->driver.dahdi.fd;
 		int dev_nr;
@@ -640,7 +646,6 @@ static int dahdi_e1_setup(struct e1inp_line *line)
 		bfd->data = line;
 		bfd->priv_nr = ts;
 		bfd->cb = dahdi_fd_cb;
-		snprintf(openstr, sizeof(openstr), "/dev/dahdi/%d", dev_nr);
 
 		switch (e1i_ts->type) {
 		case E1INP_TS_TYPE_NONE:
@@ -657,32 +662,27 @@ static int dahdi_e1_setup(struct e1inp_line *line)
 			break;
 		case E1INP_TS_TYPE_SIGN:
 			if (!bfd->fd)
-				bfd->fd = open(openstr, O_RDWR | O_NONBLOCK);
-			if (bfd->fd == -1) {
-				LOGP(DLINP, LOGL_ERROR,
-					"%s could not open %s %s\n",
-					__func__, openstr, strerror(errno));
+				bfd->fd = dahdi_open_slot(dev_nr);
+			if (bfd->fd < 0)
 				return -EIO;
-			}
 			bfd->when = BSC_FD_READ | BSC_FD_EXCEPT;
 			ret = dahdi_set_bufinfo(bfd->fd, 1);
 			if (ret < 0)
 				return ret;
 
-			if (!e1i_ts->lapd)
-				e1i_ts->lapd = lapd_instance_alloc(1,
+			if (!e1i_ts->lapd) {
+				char name[32];
+				e1inp_ts_name(name, sizeof(name), e1i_ts);
+				e1i_ts->lapd = lapd_instance_alloc2(1,
 					dahdi_write_msg, bfd, e1inp_dlsap_up,
-					e1i_ts, &lapd_profile_abis);
+					e1i_ts, &lapd_profile_abis, name);
+			}
 			break;
 		case E1INP_TS_TYPE_HDLC:
 			if (!bfd->fd)
-				bfd->fd = open(openstr, O_RDWR | O_NONBLOCK);
-			if (bfd->fd == -1) {
-				LOGP(DLINP, LOGL_ERROR,
-					"%s could not open %s %s\n",
-					__func__, openstr, strerror(errno));
+				bfd->fd = dahdi_open_slot(dev_nr);
+			if (bfd->fd < 0)
 				return -EIO;
-			}
 			bfd->when = BSC_FD_READ | BSC_FD_EXCEPT;
 			ret = dahdi_set_bufinfo(bfd->fd, 1);
 			if (ret < 0)
@@ -696,13 +696,9 @@ static int dahdi_e1_setup(struct e1inp_line *line)
 				e1i_ts->lapd = NULL;
 			}
 			if (!bfd->fd)
-				bfd->fd = open(openstr, O_RDWR | O_NONBLOCK);
-			if (bfd->fd == -1) {
-				LOGP(DLINP, LOGL_ERROR,
-					"%s could not open %s %s\n",
-					__func__, openstr, strerror(errno));
+				bfd->fd = dahdi_open_slot(dev_nr);
+			if (bfd->fd < 0)
 				return -EIO;
-			}
 			ret = dahdi_set_bufinfo(bfd->fd, 0);
 			if (ret < 0)
 				return -EIO;
@@ -713,18 +709,12 @@ static int dahdi_e1_setup(struct e1inp_line *line)
 			break;
 		}
 
-		if (bfd->fd < 0) {
-			LOGP(DLINP, LOGL_ERROR,
-				"%s could not open %s %s\n",
-				__func__, openstr, strerror(errno));
+		if (bfd->fd < 0)
 			return bfd->fd;
-		}
 
 		ret = osmo_fd_register(bfd);
 		if (ret < 0) {
-			LOGP(DLINP, LOGL_ERROR,
-				"could not register FD: %s\n",
-				strerror(ret));
+			LOGPITS(e1i_ts, DLINP, LOGL_ERROR, "could not register FD: %s\n", strerror(ret));
 			return ret;
 		}
 	}
@@ -742,10 +732,6 @@ static int dahdi_e1_line_update(struct e1inp_line *line)
 
 int e1inp_dahdi_init(void)
 {
-	init_flip_bits();
-
 	/* register the driver with the core */
 	return e1inp_driver_register(&dahdi_driver);
 }
-
-#endif /* HAVE_DAHDI_USER_H */
