@@ -47,7 +47,7 @@ void ipa_client_conn_close(struct ipa_client_conn *link)
 	link->pending_msg = NULL;
 }
 
-static void ipa_client_read(struct ipa_client_conn *link)
+static int ipa_client_read(struct ipa_client_conn *link)
 {
 	struct osmo_fd *ofd = link->ofd;
 	struct msgb *msg;
@@ -56,24 +56,21 @@ static void ipa_client_read(struct ipa_client_conn *link)
 	LOGIPA(link, LOGL_DEBUG, "message received\n");
 
 	ret = ipa_msg_recv_buffered(ofd->fd, &msg, &link->pending_msg);
-	if (ret < 0) {
+	if (ret <= 0) {
 		if (ret == -EAGAIN)
-			return;
-		if (ret == -EPIPE || ret == -ECONNRESET)
+			return 0;
+		else if (ret == -EPIPE || ret == -ECONNRESET)
 			LOGIPA(link, LOGL_ERROR, "lost connection with server\n");
+		else if (ret == 0)
+			LOGIPA(link, LOGL_ERROR, "connection closed with server\n");
 		ipa_client_conn_close(link);
 		if (link->updown_cb)
 			link->updown_cb(link, 0);
-		return;
-	} else if (ret == 0) {
-		LOGIPA(link, LOGL_ERROR, "connection closed with server\n");
-		ipa_client_conn_close(link);
-		if (link->updown_cb)
-			link->updown_cb(link, 0);
-		return;
+		return -EBADF;
 	}
 	if (link->read_cb)
-		link->read_cb(link, msg);
+		return link->read_cb(link, msg);
+	return 0;
 }
 
 static void ipa_client_write(struct ipa_client_conn *link)
@@ -115,7 +112,7 @@ static int ipa_client_write_default_cb(struct ipa_client_conn *link)
 static int ipa_client_fd_cb(struct osmo_fd *ofd, unsigned int what)
 {
 	struct ipa_client_conn *link = ofd->data;
-	int error, ret;
+	int error, ret = 0;
 	socklen_t len = sizeof(error);
 
 	switch(link->state) {
@@ -136,9 +133,9 @@ static int ipa_client_fd_cb(struct osmo_fd *ofd, unsigned int what)
 	case IPA_CLIENT_LINK_STATE_CONNECTED:
 		if (what & BSC_FD_READ) {
 			LOGIPA(link, LOGL_DEBUG, "connected read\n");
-			ipa_client_read(link);
+			ret = ipa_client_read(link);
 		}
-		if (what & BSC_FD_WRITE) {
+		if (ret != -EBADF && (what & BSC_FD_WRITE)) {
 			LOGIPA(link, LOGL_DEBUG, "connected write\n");
 			ipa_client_write(link);
 		}
@@ -157,6 +154,20 @@ ipa_client_conn_create(void *ctx, struct e1inp_ts *ts,
 				      struct msgb *msgb),
 		       int (*write_cb)(struct ipa_client_conn *link),
 		       void *data)
+{
+	return ipa_client_conn_create2(ctx, ts, priv_nr, NULL, 0, addr, port,
+				       updown_cb, read_cb, write_cb, data);
+}
+
+struct ipa_client_conn *
+ipa_client_conn_create2(void *ctx, struct e1inp_ts *ts,
+		       int priv_nr, const char *loc_addr, uint16_t loc_port,
+		       const char *rem_addr, uint16_t rem_port,
+		       void (*updown_cb)(struct ipa_client_conn *link, int up),
+		       int (*read_cb)(struct ipa_client_conn *link,
+				      struct msgb *msgb),
+		       int (*write_cb)(struct ipa_client_conn *link),
+		       void *data)
 {
 	struct ipa_client_conn *ipa_link;
 
@@ -184,8 +195,10 @@ ipa_client_conn_create(void *ctx, struct e1inp_ts *ts,
 	ipa_link->ofd->data = ipa_link;
 	ipa_link->ofd->fd = -1;
 	ipa_link->state = IPA_CLIENT_LINK_STATE_CONNECTING;
-	ipa_link->addr = talloc_strdup(ipa_link, addr);
-	ipa_link->port = port;
+	ipa_link->local_addr = talloc_strdup(ipa_link, loc_addr);
+	ipa_link->local_port = loc_port;
+	ipa_link->addr = talloc_strdup(ipa_link, rem_addr);
+	ipa_link->port = rem_port;
 	ipa_link->updown_cb = updown_cb;
 	ipa_link->read_cb = read_cb;
 	/* default to generic write callback if not set. */
@@ -212,9 +225,10 @@ int ipa_client_conn_open(struct ipa_client_conn *link)
 	int ret;
 
 	link->state = IPA_CLIENT_LINK_STATE_CONNECTING;
-	ret = osmo_sock_init(AF_INET, SOCK_STREAM, IPPROTO_TCP,
+	ret = osmo_sock_init2(AF_INET, SOCK_STREAM, IPPROTO_TCP,
+			     link->local_addr, link->local_port,
 			     link->addr, link->port,
-			     OSMO_SOCK_F_CONNECT|OSMO_SOCK_F_NONBLOCK);
+			     OSMO_SOCK_F_BIND|OSMO_SOCK_F_CONNECT|OSMO_SOCK_F_NONBLOCK);
 	if (ret < 0)
 		return ret;
 	link->ofd->fd = ret;
@@ -251,6 +265,7 @@ size_t ipa_client_conn_clear_queue(struct ipa_client_conn *link)
 static int ipa_server_fd_cb(struct osmo_fd *ofd, unsigned int what)
 {
 	int fd, ret;
+	char ipbuf[INET6_ADDRSTRLEN + 1];
 	struct sockaddr_in sa;
 	socklen_t sa_len = sizeof(sa);
 	struct ipa_server_link *link = ofd->data;
@@ -261,13 +276,20 @@ static int ipa_server_fd_cb(struct osmo_fd *ofd, unsigned int what)
 			"peer, reason=`%s'\n", strerror(errno));
 		return fd;
 	}
-	LOGP(DLINP, LOGL_NOTICE, "accept()ed new link from %s to port %u\n",
-		inet_ntoa(sa.sin_addr), link->port);
+
+	if (!link->addr) {
+		ret = osmo_sock_get_local_ip(fd, ipbuf, INET6_ADDRSTRLEN + 1);
+		if (ret == 0)
+			link->addr = talloc_strdup(link, ipbuf);
+	}
+
+	LOGIPA(link, LOGL_NOTICE, "accept()ed new link from %s:%u\n",
+		inet_ntoa(sa.sin_addr), ntohs(sa.sin_port));
 
 	ret = link->accept_cb(link, fd);
 	if (ret < 0) {
 		LOGP(DLINP, LOGL_ERROR,
-		     "failed to processs accept()ed new link, "
+		     "failed to process accept()ed new link, "
 		     "reason=`%s'\n", strerror(-ret));
 		close(fd);
 		return ret;
@@ -292,8 +314,10 @@ ipa_server_link_create(void *ctx, struct e1inp_line *line,
 
 	ipa_link->ofd.when |= BSC_FD_READ | BSC_FD_WRITE;
 	ipa_link->ofd.cb = ipa_server_fd_cb;
+	ipa_link->ofd.fd = -1;
 	ipa_link->ofd.data = ipa_link;
-	ipa_link->addr = talloc_strdup(ipa_link, addr);
+	if (addr)
+		ipa_link->addr = talloc_strdup(ipa_link, addr);
 	ipa_link->port = port;
 	ipa_link->accept_cb = accept_cb;
 	ipa_link->line = line;
@@ -320,6 +344,7 @@ int ipa_server_link_open(struct ipa_server_link *link)
 	link->ofd.fd = ret;
 	if (osmo_fd_register(&link->ofd) < 0) {
 		close(ret);
+		link->ofd.fd = -1;
 		return -EIO;
 	}
 	return 0;
@@ -327,11 +352,15 @@ int ipa_server_link_open(struct ipa_server_link *link)
 
 void ipa_server_link_close(struct ipa_server_link *link)
 {
+	if (link->ofd.fd == -1)
+		return;
+
 	osmo_fd_unregister(&link->ofd);
 	close(link->ofd.fd);
+	link->ofd.fd = -1;
 }
 
-static void ipa_server_conn_read(struct ipa_server_conn *conn)
+static int ipa_server_conn_read(struct ipa_server_conn *conn)
 {
 	struct osmo_fd *ofd = &conn->ofd;
 	struct msgb *msg;
@@ -340,22 +369,20 @@ static void ipa_server_conn_read(struct ipa_server_conn *conn)
 	LOGIPA(conn, LOGL_DEBUG, "message received\n");
 
 	ret = ipa_msg_recv_buffered(ofd->fd, &msg, &conn->pending_msg);
-	if (ret < 0) {
+	if (ret <= 0) {
 		if (ret == -EAGAIN)
-			return;
-		if (ret == -EPIPE || ret == -ECONNRESET)
+			return 0;
+		else if (ret == -EPIPE || ret == -ECONNRESET)
 			LOGIPA(conn, LOGL_ERROR, "lost connection with server\n");
+		else if (ret == 0)
+			LOGIPA(conn, LOGL_ERROR, "connection closed with server\n");
 		ipa_server_conn_destroy(conn);
-		return;
-	} else if (ret == 0) {
-		LOGIPA(conn, LOGL_ERROR, "connection closed with server\n");
-		ipa_server_conn_destroy(conn);
-		return;
+		return -EBADF;
 	}
 	if (conn->cb)
-		conn->cb(conn, msg);
+		return conn->cb(conn, msg);
 
-	return;
+	return 0;
 }
 
 static void ipa_server_conn_write(struct ipa_server_conn *conn)
@@ -381,11 +408,12 @@ static void ipa_server_conn_write(struct ipa_server_conn *conn)
 static int ipa_server_conn_cb(struct osmo_fd *ofd, unsigned int what)
 {
 	struct ipa_server_conn *conn = ofd->data;
+	int rc = 0;
 
 	LOGP(DLINP, LOGL_DEBUG, "connected read/write\n");
 	if (what & BSC_FD_READ)
-		ipa_server_conn_read(conn);
-	if (what & BSC_FD_WRITE)
+		rc = ipa_server_conn_read(conn);
+	if (rc != -EBADF && (what & BSC_FD_WRITE))
 		ipa_server_conn_write(conn);
 
 	return 0;
@@ -459,8 +487,7 @@ int ipa_server_conn_ccm(struct ipa_server_conn *conn, struct msgb *msg)
 
 	switch (msg_type) {
 	case IPAC_MSGT_ID_RESP:
-		rc = ipa_ccm_idtag_parse(&tlvp, (uint8_t *)msg->l2h + 2,
-					 msgb_l2len(msg)-2);
+		rc = ipa_ccm_id_resp_parse(&tlvp, (const uint8_t *)msg->l2h+1, msgb_l2len(msg)-1);
 		if (rc < 0) {
 			LOGIPA(conn, LOGL_ERROR, "IPA CCM RESPonse with "
 				"malformed TLVs\n");
@@ -499,7 +526,12 @@ err:
 
 void ipa_server_conn_destroy(struct ipa_server_conn *conn)
 {
+	/* make the function re-entrant in case closed_cb() below somehow
+	 * calls again into this destructor */
+	if (conn->ofd.fd == -1)
+		return;
 	close(conn->ofd.fd);
+	conn->ofd.fd = -1;
 	msgb_free(conn->pending_msg);
 	osmo_fd_unregister(&conn->ofd);
 	if (conn->closed_cb)
