@@ -59,11 +59,32 @@ static void *tall_ipa_ctx;
 #define DEFAULT_TCP_KEEPALIVE_INTERVAL     3
 #define DEFAULT_TCP_KEEPALIVE_RETRY_COUNT  10
 
+static inline struct e1inp_ts *ipaccess_line_ts(struct osmo_fd *bfd, struct e1inp_line *line)
+{
+	if (bfd->priv_nr == E1INP_SIGN_OML)
+		return e1inp_line_ipa_oml_ts(line);
+	else
+		return e1inp_line_ipa_rsl_ts(line, bfd->priv_nr - E1INP_SIGN_RSL);
+}
+
+static inline void ipaccess_keepalive_fsm_cleanup(struct e1inp_ts *e1i_ts)
+{
+	struct osmo_fsm_inst *ka_fsm;
+
+	ka_fsm = e1i_ts->driver.ipaccess.ka_fsm;
+	if (ka_fsm) {
+		ipa_keepalive_fsm_stop(ka_fsm);
+		e1i_ts->driver.ipaccess.ka_fsm = NULL;
+	}
+}
+
 static int ipaccess_drop(struct osmo_fd *bfd, struct e1inp_line *line)
 {
 	int ret = 1;
-	unsigned int ts_nr = bfd->priv_nr;
-	struct e1inp_ts *e1i_ts = &line->ts[ts_nr-1];
+	struct e1inp_ts *e1i_ts = ipaccess_line_ts(bfd, line);
+	e1inp_line_get2(line, __func__);
+
+	ipaccess_keepalive_fsm_cleanup(e1i_ts);
 
 	/* Error case: we did not see any ID_RESP yet for this socket. */
 	if (bfd->fd != -1) {
@@ -72,6 +93,13 @@ static int ipaccess_drop(struct osmo_fd *bfd, struct e1inp_line *line)
 		osmo_fd_unregister(bfd);
 		close(bfd->fd);
 		bfd->fd = -1;
+		/* This is BSC code, ipaccess_drop() is only called for
+		   accepted() sockets, hence the bfd holds a reference to
+		   e1inp_line in ->data that needs to be released */
+		OSMO_ASSERT(bfd->data == line);
+		bfd->data = NULL;
+		e1inp_line_put2(line, "ipa_bfd");
+
 		ret = -ENOENT;
 	}
 
@@ -81,9 +109,84 @@ static int ipaccess_drop(struct osmo_fd *bfd, struct e1inp_line *line)
 	/* e1inp_sign_link_destroy releases the socket descriptors for us. */
 	line->ops->sign_link_down(line);
 
+	e1inp_line_put2(line, __func__);
 	return ret;
 }
 
+static void ipa_bsc_keepalive_write_server_cb(struct osmo_fsm_inst *fi, void *conn, struct msgb *msg)
+{
+	struct osmo_fd *bfd = (struct osmo_fd *)conn;
+	write(bfd->fd, msg->data, msg->len);
+	msgb_free(msg);
+}
+
+static int ipa_bsc_keepalive_timeout_cb(struct osmo_fsm_inst *fi, void *data)
+{
+	struct osmo_fd *bfd = (struct osmo_fd *)data;
+
+	if (bfd->fd == -1)
+		return 1;
+
+	ipaccess_drop(bfd, (struct e1inp_line *)bfd->data);
+	return 1;
+}
+
+static void ipaccess_bsc_keepalive_fsm_alloc(struct e1inp_ts *e1i_ts, struct osmo_fd *bfd, const char *id)
+{
+	struct e1inp_line *line = e1i_ts->line;
+	struct osmo_fsm_inst *ka_fsm;
+
+	ipaccess_keepalive_fsm_cleanup(e1i_ts);
+	if (!line->ipa_kap)
+		return;
+
+	ka_fsm = ipa_generic_conn_alloc_keepalive_fsm(tall_ipa_ctx, bfd, line->ipa_kap, id);
+	e1i_ts->driver.ipaccess.ka_fsm = ka_fsm;
+	if (!ka_fsm)
+		return;
+
+	ipa_keepalive_fsm_set_timeout_cb(ka_fsm, ipa_bsc_keepalive_timeout_cb);
+	ipa_keepalive_fsm_set_send_cb(ka_fsm, ipa_bsc_keepalive_write_server_cb);
+	ipa_keepalive_fsm_start(ka_fsm);
+}
+
+static void ipa_bts_keepalive_write_client_cb(struct osmo_fsm_inst *fi, void *conn, struct msgb *msg) {
+	struct ipa_client_conn *link = (struct ipa_client_conn *)conn;
+	int ret = 0;
+
+	ret = ipa_send(link->ofd->fd, msg->data, msg->len);
+	if (ret != msg->len) {
+		LOGP(DLINP, LOGL_ERROR, "cannot send message. Reason: %s\n", strerror(errno));
+	}
+	msgb_free(msg);
+}
+
+static void update_fd_settings(struct e1inp_line *line, int fd);
+static void ipaccess_bts_updown_cb(struct ipa_client_conn *link, int up);
+
+static int ipa_bts_keepalive_timeout_cb(struct osmo_fsm_inst *fi, void *conn) {
+	ipaccess_bts_updown_cb(conn, false);
+	return 1;
+}
+
+static void ipaccess_bts_keepalive_fsm_alloc(struct e1inp_ts *e1i_ts, struct ipa_client_conn *client, const char *id)
+{
+	struct e1inp_line *line = e1i_ts->line;
+	struct osmo_fsm_inst *ka_fsm;
+
+	ipaccess_keepalive_fsm_cleanup(e1i_ts);
+	if (!line->ipa_kap)
+		return;
+
+	ka_fsm = ipa_client_conn_alloc_keepalive_fsm(client, line->ipa_kap, id);
+	e1i_ts->driver.ipaccess.ka_fsm = ka_fsm;
+	if (!ka_fsm)
+		return;
+
+	ipa_keepalive_fsm_set_timeout_cb(ka_fsm, ipa_bts_keepalive_timeout_cb);
+	ipa_keepalive_fsm_set_send_cb(ka_fsm, ipa_bts_keepalive_write_client_cb);
+}
+
 /* Returns -1 on error, and 0 or 1 on success. If -1 or 1 is returned, line has
  * been released and should not be used anymore by the caller. */
 static int ipaccess_rcvmsg(struct e1inp_line *line, struct msgb *msg,
@@ -95,6 +198,14 @@ static int ipaccess_rcvmsg(struct e1inp_line *line, struct msgb *msg,
 	struct e1inp_sign_link *sign_link;
 	char *unitid;
 	int len, ret;
+	struct e1inp_ts *e1i_ts;
+	struct osmo_fsm_inst *ka_fsm;
+
+	/* peek the pong for our keepalive fsm */
+	e1i_ts = ipaccess_line_ts(bfd, line);
+	ka_fsm = e1i_ts->driver.ipaccess.ka_fsm;
+	if (ka_fsm && msg_type == IPAC_MSGT_PONG)
+		ipa_keepalive_fsm_pong_received(ka_fsm);
 
 	/* Handle IPA PING, PONG and ID_ACK messages. */
 	ret = ipa_ccm_rcvmsg_base(msg, bfd);
@@ -117,11 +228,10 @@ static int ipaccess_rcvmsg(struct e1inp_line *line, struct msgb *msg,
 
 	switch (msg_type) {
 	case IPAC_MSGT_ID_RESP:
-		DEBUGP(DLMI, "ID_RESP\n");
+		DEBUGP(DLMI, "ID_RESP ");
 		/* parse tags, search for Unit ID */
-		ret = ipa_ccm_idtag_parse(&tlvp, (uint8_t *)msg->l2h + 2,
-						msgb_l2len(msg)-2);
-		DEBUGP(DLMI, "\n");
+		ret = ipa_ccm_id_resp_parse(&tlvp, (const uint8_t *)msg->l2h+1, msgb_l2len(msg)-1);
+		DEBUGPC(DLMI, "\n");
 		if (ret < 0) {
 			LOGP(DLINP, LOGL_ERROR, "IPA response message "
 				"with malformed TLVs\n");
@@ -141,7 +251,11 @@ static int ipaccess_rcvmsg(struct e1inp_line *line, struct msgb *msg,
 		}
 		unitid = (char *) TLVP_VAL(&tlvp, IPAC_IDTAG_UNIT);
 		unitid[len - 1] = '\0';
-		ipa_parse_unitid(unitid, &unit_data);
+		ret = ipa_parse_unitid(unitid, &unit_data);
+		if (ret) {
+			LOGP(DLINP, LOGL_ERROR, "Failed to parse unit ID '%s'\n", unitid);
+			goto err;
+		}
 
 		if (!line->ops->sign_link_up) {
 			LOGP(DLINP, LOGL_ERROR,
@@ -159,9 +273,12 @@ static int ipaccess_rcvmsg(struct e1inp_line *line, struct msgb *msg,
 					"closing socket.\n");
 				goto err;
 			}
+
+			ipaccess_bsc_keepalive_fsm_alloc(e1i_ts, bfd, "oml_bsc_to_bts");
+
 		} else if (bfd->priv_nr == E1INP_SIGN_RSL) {
 			struct e1inp_ts *ts;
-                        struct osmo_fd *newbfd;
+			struct osmo_fd *newbfd;
 			struct e1inp_line *new_line;
 
 			sign_link =
@@ -173,38 +290,45 @@ static int ipaccess_rcvmsg(struct e1inp_line *line, struct msgb *msg,
 					"closing socket.\n");
 				goto err;
 			}
+			/* Finally, we know which OML link is associated with
+			 * this RSL link, attach it to this socket. */
+			new_line = sign_link->ts->line;
 			/* this is a bugtrap, the BSC should be using the
 			 * virtual E1 line used by OML for this RSL link. */
-			if (sign_link->ts->line == line) {
+			if (new_line == line) {
 				LOGP(DLINP, LOGL_ERROR,
 					"Fix your BSC, you should use the "
 					"E1 line used by the OML link for "
 					"your RSL link.\n");
 				return 0;
 			}
-			/* Finally, we know which OML link is associated with
-			 * this RSL link, attach it to this socket. */
-			bfd->data = new_line = sign_link->ts->line;
-			e1inp_line_get(new_line);
-			ts = &new_line->ts[E1INP_SIGN_RSL+unit_data.trx_id-1];
+			e1inp_line_get2(new_line, "ipa_bfd");
+			ts = e1inp_line_ipa_rsl_ts(new_line, unit_data.trx_id);
 			newbfd = &ts->driver.ipaccess.fd;
+			OSMO_ASSERT(newbfd != bfd);
 
-			/* get rid of our old temporary bfd */
-			newbfd->fd = bfd->fd;
-			newbfd->when |= bfd->when; /* preserve 'newbfd->when' flags potentially set by sign_link_up() */
-			newbfd->cb = bfd->cb;
-			newbfd->data = bfd->data;
-			newbfd->priv_nr = E1INP_SIGN_RSL + unit_data.trx_id;
+			/* preserve 'newbfd->when' flags potentially set by sign_link_up() */
+			osmo_fd_setup(newbfd, bfd->fd, newbfd->when | bfd->when, bfd->cb,
+				      new_line, E1INP_SIGN_RSL + unit_data.trx_id);
+
+
+			/* now we can release the dummy RSL line (old temporary bfd). */
 			osmo_fd_unregister(bfd);
 			bfd->fd = -1;
+			/* bfd->data holds a reference to line, drop it */
+			OSMO_ASSERT(bfd->data == line);
+			bfd->data = NULL;
+			e1inp_line_put2(line, "ipa_bfd");
+
 			ret = osmo_fd_register(newbfd);
 			if (ret < 0) {
 				LOGP(DLINP, LOGL_ERROR,
 				     "could not register FD\n");
 				goto err;
 			}
-			/* now we can release the dummy RSL line. */
-			e1inp_line_put(line);
+
+			e1i_ts = ipaccess_line_ts(newbfd, new_line);
+			ipaccess_bsc_keepalive_fsm_alloc(e1i_ts, newbfd, "rsl_bsc_to_bts");
 			return 1;
 		}
 		break;
@@ -214,10 +338,15 @@ static int ipaccess_rcvmsg(struct e1inp_line *line, struct msgb *msg,
 	}
 	return 0;
 err:
-	osmo_fd_unregister(bfd);
-	close(bfd->fd);
-	bfd->fd = -1;
-	e1inp_line_put(line);
+	if (bfd->fd != -1) {
+		osmo_fd_unregister(bfd);
+		close(bfd->fd);
+		bfd->fd = -1;
+		/* This is a BSC accepted socket, bfd->data holds a reference to line, drop it */
+		OSMO_ASSERT(bfd->data == line);
+		bfd->data = NULL;
+		e1inp_line_put2(line, "ipa_bfd");
+	}
 	return -1;
 }
 
@@ -226,12 +355,13 @@ static int handle_ts1_read(struct osmo_fd *bfd)
 {
 	struct e1inp_line *line = bfd->data;
 	unsigned int ts_nr = bfd->priv_nr;
-	struct e1inp_ts *e1i_ts = &line->ts[ts_nr-1];
+	struct e1inp_ts *e1i_ts;
 	struct e1inp_sign_link *link;
 	struct ipaccess_head *hh;
 	struct msgb *msg = NULL;
 	int ret, rc;
 
+	e1i_ts = ipaccess_line_ts(bfd, line);
 	ret = ipa_msg_recv_buffered(bfd->fd, &msg, &e1i_ts->pending_msg);
 	if (ret < 0) {
 		if (ret == -EAGAIN)
@@ -282,7 +412,7 @@ static int handle_ts1_read(struct osmo_fd *bfd)
 		     " sign_link returned error: %s.\n", strerror(-rc));
 	}
 
-	return 0;
+	return rc;
 err_msg:
 	msgb_free(msg);
 err:
@@ -301,7 +431,30 @@ static void ipaccess_close(struct e1inp_sign_link *sign_link)
 {
 	struct e1inp_ts *e1i_ts = sign_link->ts;
 	struct osmo_fd *bfd = &e1i_ts->driver.ipaccess.fd;
-	return e1inp_close_socket(e1i_ts, sign_link, bfd);
+	struct e1inp_line *line = e1i_ts->line;
+
+	/* line might not exist if != bsc||bts */
+	if (line) {
+		/* depending on caller the fsm might be dead */
+		struct osmo_fsm_inst* ka_fsm = ipaccess_line_ts(bfd, line)->driver.ipaccess.ka_fsm;
+		if (ka_fsm)
+			ipa_keepalive_fsm_stop(ka_fsm);
+
+	}
+
+	e1inp_int_snd_event(e1i_ts, sign_link, S_L_INP_TEI_DN);
+	/* the first e1inp_sign_link_destroy call closes the socket. */
+	if (bfd->fd != -1) {
+		osmo_fd_unregister(bfd);
+		close(bfd->fd);
+		bfd->fd = -1;
+		/* If The bfd holds a reference to e1inp_line in ->data (BSC
+		 * accepted() sockets), then release it */
+		if (bfd->data == line) {
+			bfd->data = NULL;
+			e1inp_line_put2(line, "ipa_bfd");
+		}
+	}
 }
 
 static void timeout_ts1_write(void *data)
@@ -315,11 +468,12 @@ static void timeout_ts1_write(void *data)
 static int __handle_ts1_write(struct osmo_fd *bfd, struct e1inp_line *line)
 {
 	unsigned int ts_nr = bfd->priv_nr;
-	struct e1inp_ts *e1i_ts = &line->ts[ts_nr-1];
+	struct e1inp_ts *e1i_ts;
 	struct e1inp_sign_link *sign_link;
 	struct msgb *msg;
 	int ret;
 
+	e1i_ts = ipaccess_line_ts(bfd, line);
 	bfd->when &= ~BSC_FD_WRITE;
 
 	/* get the next msg for this timeslot */
@@ -449,42 +603,53 @@ static void update_fd_settings(struct e1inp_line *line, int fd)
 			LOGP(DLINP, LOGL_NOTICE,
 			     "Failed to set keepalive count: %s\n",
 			     strerror(errno));
+#if defined(TCP_USER_TIMEOUT)
+                val = 1000 * line->keepalive_num_probes *
+                        line->keepalive_probe_interval +
+                        line->keepalive_idle_timeout;
+                ret = setsockopt(fd, IPPROTO_TCP, TCP_USER_TIMEOUT,
+                                 &val, sizeof(val));
+                if (ret < 0)
+                        LOGP(DLINP, LOGL_NOTICE,
+                             "Failed to set user timoeut: %s\n",
+                             strerror(errno));
+#endif
 #endif
 	}
+
+	val = 1;
+	ret = setsockopt(fd, IPPROTO_TCP, TCP_NODELAY, &val, sizeof(val));
+	if (ret < 0)
+		LOGP(DLINP, LOGL_ERROR, "Failed to set TCP_NODELAY: %s\n", strerror(errno));
 }
 
 /* callback of the OML listening filedescriptor */
 static int ipaccess_bsc_oml_cb(struct ipa_server_link *link, int fd)
 {
 	int ret;
-	int idx = 0;
 	int i;
 	struct e1inp_line *line;
 	struct e1inp_ts *e1i_ts;
 	struct osmo_fd *bfd;
 
 	/* clone virtual E1 line for this new OML link. */
-	line = e1inp_line_clone(tall_ipa_ctx, link->line);
+	line = e1inp_line_clone(tall_ipa_ctx, link->line, "ipa_bfd");
 	if (line == NULL) {
 		LOGP(DLINP, LOGL_ERROR, "could not clone E1 line\n");
 		return -ENOMEM;
 	}
 
 	/* create virrtual E1 timeslots for signalling */
-	e1inp_ts_config_sign(&line->ts[E1INP_SIGN_OML-1], line);
+	e1inp_ts_config_sign(e1inp_line_ipa_oml_ts(line), line);
 
 	/* initialize the fds */
 	for (i = 0; i < ARRAY_SIZE(line->ts); ++i)
 		line->ts[i].driver.ipaccess.fd.fd = -1;
 
-	e1i_ts = &line->ts[idx];
+	e1i_ts = e1inp_line_ipa_oml_ts(line);
 
 	bfd = &e1i_ts->driver.ipaccess.fd;
-	bfd->fd = fd;
-	bfd->data = line;
-	bfd->priv_nr = E1INP_SIGN_OML;
-	bfd->cb = ipaccess_fd_cb;
-	bfd->when = BSC_FD_READ;
+	osmo_fd_setup(bfd, fd, BSC_FD_READ, ipaccess_fd_cb, line, E1INP_SIGN_OML);
 	ret = osmo_fd_register(bfd);
 	if (ret < 0) {
 		LOGP(DLINP, LOGL_ERROR, "could not register FD\n");
@@ -507,7 +672,8 @@ err_socket:
 err_line:
 	close(bfd->fd);
 	bfd->fd = -1;
-	e1inp_line_put(line);
+	bfd->data = NULL;
+	e1inp_line_put2(line, "ipa_bfd");
 	return ret;
 }
 
@@ -520,7 +686,7 @@ static int ipaccess_bsc_rsl_cb(struct ipa_server_link *link, int fd)
 
         /* We don't know yet which OML link to associate it with. Thus, we
          * allocate a temporary E1 line until we have received ID. */
-	line = e1inp_line_clone(tall_ipa_ctx, link->line);
+	line = e1inp_line_clone(tall_ipa_ctx, link->line, "ipa_bfd");
 	if (line == NULL) {
 		LOGP(DLINP, LOGL_ERROR, "could not clone E1 line\n");
 		return -ENOMEM;
@@ -531,16 +697,12 @@ static int ipaccess_bsc_rsl_cb(struct ipa_server_link *link, int fd)
 
 	/* we need this to initialize this in case to avoid crashes in case
 	 * that the socket is closed before we've seen an ID_RESP. */
-	e1inp_ts_config_sign(&line->ts[E1INP_SIGN_OML-1], line);
+	e1inp_ts_config_sign(e1inp_line_ipa_oml_ts(line), line);
 
-	e1i_ts = &line->ts[E1INP_SIGN_RSL-1];
+	e1i_ts = e1inp_line_ipa_rsl_ts(line, 0);
 
 	bfd = &e1i_ts->driver.ipaccess.fd;
-	bfd->fd = fd;
-	bfd->data = line;
-	bfd->priv_nr = E1INP_SIGN_RSL;
-	bfd->cb = ipaccess_fd_cb;
-	bfd->when = BSC_FD_READ;
+	osmo_fd_setup(bfd, fd, BSC_FD_READ, ipaccess_fd_cb, line, E1INP_SIGN_RSL);
 	ret = osmo_fd_register(bfd);
 	if (ret < 0) {
 		LOGP(DLINP, LOGL_ERROR, "could not register FD\n");
@@ -561,7 +723,8 @@ err_socket:
 err_line:
 	close(bfd->fd);
 	bfd->fd = -1;
-	e1inp_line_put(line);
+	bfd->data = NULL;
+	e1inp_line_put2(line, "ipa_bfd");
 	return ret;
 }
 
@@ -602,19 +765,19 @@ ipa_bts_id_resp(const struct ipaccess_unit *dev, uint8_t *data, int len, int trx
 			break;
 		case IPAC_IDTAG_LOCATION1:
 			if (dev->location1)
-				strncpy(str, dev->location1, IPA_STRING_MAX);
+				osmo_strlcpy(str, dev->location1, sizeof(str));
 			break;
 		case IPAC_IDTAG_LOCATION2:
 			if (dev->location2)
-				strncpy(str, dev->location2, IPA_STRING_MAX);
+				osmo_strlcpy(str, dev->location2, sizeof(str));
 			break;
 		case IPAC_IDTAG_EQUIPVERS:
 			if (dev->equipvers)
-				strncpy(str, dev->equipvers, IPA_STRING_MAX);
+				osmo_strlcpy(str, dev->equipvers, sizeof(str));
 			break;
 		case IPAC_IDTAG_SWVERSION:
 			if (dev->swversion)
-				strncpy(str, dev->swversion, IPA_STRING_MAX);
+				osmo_strlcpy(str, dev->swversion, sizeof(str));
 			break;
 		case IPAC_IDTAG_UNITNAME:
 			snprintf(str, sizeof(str),
@@ -626,7 +789,7 @@ ipa_bts_id_resp(const struct ipaccess_unit *dev, uint8_t *data, int len, int trx
 			break;
 		case IPAC_IDTAG_SERNR:
 			if (dev->serno)
-				strncpy(str, dev->serno, IPA_STRING_MAX);
+				osmo_strlcpy(str, dev->serno, sizeof(str));
 			break;
 		default:
 			LOGP(DLINP, LOGL_NOTICE,
@@ -634,7 +797,6 @@ ipa_bts_id_resp(const struct ipaccess_unit *dev, uint8_t *data, int len, int trx
 			msgb_free(nmsg);
 			return NULL;
 		}
-		str[IPA_STRING_MAX-1] = '\0';
 
 		LOGP(DLINP, LOGL_INFO, " tag %d: %s\n", data[1], str);
 		tag = msgb_put(nmsg, 3 + strlen(str) + 1);
@@ -667,8 +829,14 @@ static void ipaccess_bts_updown_cb(struct ipa_client_conn *link, int up)
 {
 	struct e1inp_line *line = link->line;
 
-	if (up)
+	if (up) {
+		struct osmo_fsm_inst *ka_fsm = ipaccess_line_ts(link->ofd, line)->driver.ipaccess.ka_fsm;
+
+		update_fd_settings(line, link->ofd->fd);
+		if (ka_fsm && line->ipa_kap)
+			ipa_keepalive_fsm_start(ka_fsm);
 		return;
+	}
 
 	if (line->ops->sign_link_down)
 		line->ops->sign_link_down(line);
@@ -682,10 +850,19 @@ int ipaccess_bts_handle_ccm(struct ipa_client_conn *link,
 	struct ipaccess_head *hh = (struct ipaccess_head *) msg->data;
 	struct msgb *rmsg;
 	int ret = 0;
+	/* line might not exist if != bsc||bts */
+	struct e1inp_line *line = link->line;
 
 	/* special handling for IPA CCM. */
 	if (hh->proto == IPAC_PROTO_IPACCESS) {
 		uint8_t msg_type = *(msg->l2h);
+		struct osmo_fsm_inst* ka_fsm = NULL;
+
+		/* peek the pong for our keepalive fsm */
+		if (line && msg_type == IPAC_MSGT_PONG) {
+			ka_fsm = ipaccess_line_ts(link->ofd, line)->driver.ipaccess.ka_fsm;
+			ipa_keepalive_fsm_pong_received(ka_fsm);
+		}
 
 		/* ping, pong and acknowledgment cases. */
 		ret = ipa_ccm_rcvmsg_bts_base(msg, link->ofd);
@@ -701,8 +878,8 @@ int ipaccess_bts_handle_ccm(struct ipa_client_conn *link,
 			if (link->ofd->priv_nr >= E1INP_SIGN_RSL)
 				trx_nr = link->ofd->priv_nr - E1INP_SIGN_RSL;
 
-			LOGP(DLINP, LOGL_NOTICE, "received ID get from %u/%u/%u\n",
-			     dev->site_id, dev->bts_id, dev->trx_id);
+			LOGP(DLINP, LOGL_NOTICE, "received ID_GET for unit ID %u/%u/%u\n",
+			     dev->site_id, dev->bts_id, trx_nr);
 			rmsg = ipa_bts_id_resp(dev, data + 1, len - 1, trx_nr);
 			ret = ipa_send(link->ofd->fd, rmsg->data, rmsg->len);
 			if (ret != rmsg->len) {
@@ -739,19 +916,17 @@ static int ipaccess_bts_read_cb(struct ipa_client_conn *link, struct msgb *msg)
 	struct ipaccess_head *hh = (struct ipaccess_head *) msg->data;
 	struct e1inp_ts *e1i_ts = NULL;
 	struct e1inp_sign_link *sign_link;
+	uint8_t msg_type = *(msg->l2h);
 	int ret = 0;
 
 	/* special handling for IPA CCM. */
 	if (hh->proto == IPAC_PROTO_IPACCESS) {
-		uint8_t msg_type = *(msg->l2h);
-
 		/* this is a request for identification from the BSC. */
 		if (msg_type == IPAC_MSGT_ID_GET) {
 			if (!link->line->ops->sign_link_up) {
 				LOGP(DLINP, LOGL_ERROR,
 					"Unable to set signal link, "
 					"closing socket.\n");
-				ret = -EINVAL;
 				goto err;
 			}
 		}
@@ -763,8 +938,6 @@ static int ipaccess_bts_read_cb(struct ipa_client_conn *link, struct msgb *msg)
 		goto err;
 
 	if (ret == 1 && hh->proto == IPAC_PROTO_IPACCESS) {
-		uint8_t msg_type = *(msg->l2h);
-
 		if (msg_type == IPAC_MSGT_ID_GET) {
 			sign_link = link->line->ops->sign_link_up(msg,
 					link->line,
@@ -773,25 +946,29 @@ static int ipaccess_bts_read_cb(struct ipa_client_conn *link, struct msgb *msg)
 				LOGP(DLINP, LOGL_ERROR,
 					"Unable to set signal link, "
 					"closing socket.\n");
-				ret = -EINVAL;
 				goto err;
 			}
 		}
 		msgb_free(msg);
 		return ret;
 	} else if (link->port == IPA_TCP_PORT_OML)
-		e1i_ts = &link->line->ts[0];
+		e1i_ts = e1inp_line_ipa_oml_ts(link->line);
 	else if (link->port == IPA_TCP_PORT_RSL)
-		e1i_ts = &link->line->ts[link->ofd->priv_nr-1];
+		e1i_ts = e1inp_line_ipa_rsl_ts(link->line, link->ofd->priv_nr - E1INP_SIGN_RSL);
 
 	OSMO_ASSERT(e1i_ts != NULL);
 
+	if (e1i_ts->type == E1INP_TS_TYPE_NONE) {
+		LOGP(DLINP, LOGL_ERROR, "Signalling link not initialized. Discarding."
+		     " port=%u msg_type=%u\n", link->port, msg_type);
+		goto err;
+	}
+
 	/* look up for some existing signaling link. */
 	sign_link = e1inp_lookup_sign_link(e1i_ts, hh->proto, 0);
 	if (sign_link == NULL) {
 		LOGP(DLINP, LOGL_ERROR, "no matching signalling link for "
 			"hh->proto=0x%02x\n", hh->proto);
-		ret = -EIO;
 		goto err;
 	}
 	msg->dst = sign_link;
@@ -800,20 +977,18 @@ static int ipaccess_bts_read_cb(struct ipa_client_conn *link, struct msgb *msg)
 	if (!link->line->ops->sign_link) {
 		LOGP(DLINP, LOGL_ERROR, "Fix your application, "
 			"no action set for signalling messages.\n");
-		ret = -ENOENT;
 		goto err;
 	}
-	link->line->ops->sign_link(msg);
-	return 0;
+	return link->line->ops->sign_link(msg);
 
 err:
 	ipa_client_conn_close(link);
 	msgb_free(msg);
-	return ret;
+	return -EBADF;
 }
 
 struct ipaccess_line {
-	int line_already_initialized;
+	bool line_already_initialized;
 };
 
 static int ipaccess_line_update(struct e1inp_line *line)
@@ -834,7 +1009,7 @@ static int ipaccess_line_update(struct e1inp_line *line)
 	if (il->line_already_initialized)
 		return 0;
 
-	il->line_already_initialized = 1;
+	il->line_already_initialized = true;
 
 	switch(line->ops->cfg.ipa.role) {
 	case E1INP_LINE_R_BSC: {
@@ -878,14 +1053,16 @@ static int ipaccess_line_update(struct e1inp_line *line)
 	}
 	case E1INP_LINE_R_BTS: {
 		struct ipa_client_conn *link;
+		struct e1inp_ts *e1i_ts;
 
 		LOGP(DLINP, LOGL_NOTICE, "enabling ipaccess BTS mode, "
 		     "OML connecting to %s:%u\n", line->ops->cfg.ipa.addr,
 		     IPA_TCP_PORT_OML);
 
-		link = ipa_client_conn_create(tall_ipa_ctx,
-					      &line->ts[E1INP_SIGN_OML-1],
+		link = ipa_client_conn_create2(tall_ipa_ctx,
+					      e1inp_line_ipa_oml_ts(line),
 					      E1INP_SIGN_OML,
+					      NULL, 0,
 					      line->ops->cfg.ipa.addr,
 					      IPA_TCP_PORT_OML,
 					      ipaccess_bts_updown_cb,
@@ -904,6 +1081,9 @@ static int ipaccess_line_update(struct e1inp_line *line)
 			ipa_client_conn_destroy(link);
 			return -EIO;
 		}
+
+		e1i_ts = e1inp_line_ipa_oml_ts(line);
+		ipaccess_bts_keepalive_fsm_alloc(e1i_ts, link, "oml_bts_to_bsc");
 		ret = 0;
 		break;
 	}
@@ -926,6 +1106,7 @@ int e1inp_ipa_bts_rsl_connect_n(struct e1inp_line *line,
 				uint8_t trx_nr)
 {
 	struct ipa_client_conn *rsl_link;
+	struct e1inp_ts *e1i_ts = e1inp_line_ipa_rsl_ts(line, trx_nr);
 
 	if (E1INP_SIGN_RSL+trx_nr-1 >= NUM_E1_TS) {
 		LOGP(DLINP, LOGL_ERROR, "cannot create RSL BTS link: "
@@ -933,9 +1114,10 @@ int e1inp_ipa_bts_rsl_connect_n(struct e1inp_line *line,
 		return -EINVAL;
 	}
 
-	rsl_link = ipa_client_conn_create(tall_ipa_ctx,
-					  &line->ts[E1INP_SIGN_RSL+trx_nr-1],
+	rsl_link = ipa_client_conn_create2(tall_ipa_ctx,
+					  e1inp_line_ipa_rsl_ts(line, trx_nr),
 					  E1INP_SIGN_RSL+trx_nr,
+					  NULL, 0,
 					  rem_addr, rem_port,
 					  ipaccess_bts_updown_cb,
 					  ipaccess_bts_read_cb,
@@ -953,6 +1135,8 @@ int e1inp_ipa_bts_rsl_connect_n(struct e1inp_line *line,
 		ipa_client_conn_destroy(rsl_link);
 		return -EIO;
 	}
+
+	ipaccess_bts_keepalive_fsm_alloc(e1i_ts, rsl_link, "rsl_bts_to_bsc");
 	return 0;
 }
 
