@@ -184,7 +184,8 @@ ipa_sock_src_accept_cb(struct ipa_server_link *link, int fd)
 
 	LOGP(DLINP, LOGL_NOTICE, "now trying to connect to destination\n");
 
-	conn->dst = ipa_client_conn_create(NULL, NULL, 0,
+	conn->dst = ipa_client_conn_create2(NULL, NULL, 0,
+					   NULL, 0,
 					   route->shared->dst.inst->net.addr,
 					   route->shared->dst.inst->net.port,
 					   NULL,
@@ -245,8 +246,7 @@ static int __ipa_instance_add(struct vty *vty, int argc, const char *argv[])
 			VTY_NEWLINE);
 		return CMD_WARNING;
 	}
-	strncpy(ipi->name, argv[0], IPA_INSTANCE_NAME);
-	ipi->name[IPA_INSTANCE_NAME - 1] = '\0';
+	osmo_strlcpy(ipi->name, argv[0], sizeof(ipi->name));
 	ipi->net.type = type;
 	ipi->net.addr = talloc_strdup(tall_ipa_proxy_ctx, argv[2]);
 	ipi->net.port = port;
