@@ -40,7 +40,7 @@
 
 void *tall_tqe_ctx;
 
-static inline void append_bit(struct demux_subch *sch, uint8_t bit)
+static inline void append_bit(struct demux_subch *sch, ubit_t bit)
 {
 	sch->out_bitbuf[sch->out_idx++] = bit;
 }
@@ -92,6 +92,29 @@ int subch_demux_init(struct subch_demux *dmx)
 	return 0;
 }
 
+static void append_bit_resync_out(struct subch_demux *dmx, int c, ubit_t bit)
+{
+	struct demux_subch *sch = &dmx->subch[c];
+	append_bit(sch, bit);
+
+	if (sync_hdr_complete(sch, bit))
+		resync_to_here(sch);
+
+	/* FIXME: verify the first bit in octet 2, 4, 6, ...
+	 * according to TS 08.60 4.8.1 */
+
+	/* once we have reached TRAU_FRAME_BITS, call
+	 * the TRAU frame handler callback function */
+	if (sch->out_idx >= TRAU_FRAME_BITS) {
+		if (sch->in_sync) {
+			dmx->out_cb(dmx, c, sch->out_bitbuf,
+			    sch->out_idx, dmx->data);
+			sch->in_sync = 0;
+		}
+		sch->out_idx = 0;
+	}
+}
+
 /*! \brief Input some data from the 64k full-slot into subchannel demux
  *  \param[in] dmx subchannel demuxer
  *  \param[in] data pointer to buffer containing input data
@@ -108,7 +131,6 @@ int subch_demux_in(struct subch_demux *dmx, uint8_t *data, int len)
 		uint8_t inbyte = data[i];
 
 		for (c = 0; c < NR_SUBCH; c++) {
-			struct demux_subch *sch = &dmx->subch[c];
 			uint8_t inbits;
 			uint8_t bit;
 
@@ -123,33 +145,13 @@ int subch_demux_in(struct subch_demux *dmx, uint8_t *data, int len)
 				bit = 1;
 			else
 				bit = 0;
-			append_bit(sch, bit);
-
-			if (sync_hdr_complete(sch, bit))
-				resync_to_here(sch);
+			append_bit_resync_out(dmx, c, bit);
 
 			if (inbits & 0x02)
 				bit = 1;
 			else
 				bit = 0;
-			append_bit(sch, bit);
-
-			if (sync_hdr_complete(sch, bit))
-				resync_to_here(sch);
-
-			/* FIXME: verify the first bit in octet 2, 4, 6, ...
-			 * according to TS 08.60 4.8.1 */
-
-			/* once we have reached TRAU_FRAME_BITS, call
-			 * the TRAU frame handler callback function */
-			if (sch->out_idx >= TRAU_FRAME_BITS) {
-				if (sch->in_sync) {
-					dmx->out_cb(dmx, c, sch->out_bitbuf,
-					    sch->out_idx, dmx->data);
-					sch->in_sync = 0;
-				}
-				sch->out_idx = 0;
-			}
+			append_bit_resync_out(dmx, c, bit);
 		}
 	}
 	return i;
@@ -200,7 +202,7 @@ static int alloc_add_idle_frame(struct subch_mux *mx, int sch_nr)
 
 /* return the requested number of bits from the specified subchannel */
 static int get_subch_bits(struct subch_mux *mx, int subch,
-			  uint8_t *bits, int num_requested)
+			  ubit_t *bits, int num_requested)
 {
 	struct mux_subch *sch = &mx->subch[subch];
 	int num_bits = 0;
@@ -258,7 +260,7 @@ static uint8_t compact_bits(const uint8_t *bits)
 /* obtain a single output byte from the subchannel muxer */
 static int mux_output_byte(struct subch_mux *mx, uint8_t *byte)
 {
-	uint8_t bits[8];
+	ubit_t bits[8];
 	int rc;
 
 	/* combine two bits of every subchan */
@@ -310,11 +312,11 @@ static void tx_queue_evict(struct mux_subch *sch, int num_evict)
 /*! \brief enqueue some data into the tx_queue of a given subchannel
  *  \param[in] mx subchannel muxer instance
  *  \param[in] s_nr subchannel number
- *  \param[in] data pointer to buffer with data
- *  \param[in] len length of \a data
+ *  \param[in] data pointer to buffer with data (unpacked bits)
+ *  \param[in] len length of data (in unpacked bits)
  *  \returns 0 in case of success, <0 in case of error
  */
-int subchan_mux_enqueue(struct subch_mux *mx, int s_nr, const uint8_t *data,
+int subchan_mux_enqueue(struct subch_mux *mx, int s_nr, const ubit_t *data,
 			int len)
 {
 	struct mux_subch *sch = &mx->subch[s_nr];
