@@ -102,8 +102,35 @@ static void my_ortp_logfn(
 #endif
 	OrtpLogLevel lev, const char *fmt, va_list args)
 {
+	/* Some strings coming from ortp are not endline terminated and mangle
+	 * the output. Make sure all strings are endl terminated before
+	 * printing.
+	 */
+	int needs_endl;
+	const char *domain_str;
+	char *str;
+	size_t fmt_len = strlen(fmt);
+#if HAVE_ORTP_LOG_DOMAIN
+	/* domain can be NULL, found experimentally */
+	domain_str = domain ? : "";
+#else
+	domain_str = "";
+#endif
+	size_t  domain_len = strlen(domain_str);
+
+	if (fmt_len == 0)
+		return;
+
+	needs_endl = fmt[fmt_len - 1] != '\n' ? 1 : 0;
+
+	str = osmo_ortp_malloc(domain_len + 2 /*": "*/ + fmt_len + needs_endl + 1);
+	sprintf(str, "%s%s%s%s", domain_str, domain_len ? ": " : "", fmt, needs_endl ? "\n" : "");
+
 	osmo_vlogp(DLMIB, ortp_to_osmo_lvl(lev), __FILE__, 0,
-		   0, fmt, args);
+		   0, str, args);
+
+	osmo_ortp_free(str);
+
 }
 
 /* ORTP signal callbacks */
@@ -351,6 +378,12 @@ struct osmo_rtp_socket *osmo_rtp_socket_create(void *talloc_ctx, unsigned int fl
 	rtp_session_set_profile(rs->sess, osmo_pt_profile);
 	rtp_session_set_jitter_compensation(rs->sess, 100);
 
+	/* ortp >= 0.24.0 doesn't differentiate between SO_REUSEADDR and
+	 * SO_REUSEPORT, and has both enabled by default.  The latter means that
+	 * we can end up with non-unique port bindings as we will not fail to
+	 * bind the same port twice */
+	rtp_session_set_reuseaddr(rs->sess, false);
+
 	rtp_session_signal_connect(rs->sess, "ssrc_changed",
 				   (RtpCallback) ortp_sig_cb_ssrc,
 				   RTP_SIGNAL_PTR_CAST(rs));
@@ -441,6 +474,28 @@ int osmo_rtp_socket_connect(struct osmo_rtp_socket *rs, const char *ip, uint16_t
 		return osmo_rtp_socket_fdreg(rs);
 }
 
+/*! \brief Automatically associates a RTP socket with the first incoming UDP packet
+ *  \param[in] rs OsmoRTP socket
+ *
+ * The bound RTP socket will wait for incoming RTP packets and as soon as it
+ * sees one, will 'connect' to it, so all replies will go to that sources and
+ * incoming messages from other sources will be discarded. This obviously only
+ * works once.
+ *
+ *  \returns 0 on success, <0 in case of error.
+ */
+int osmo_rtp_socket_autoconnect(struct osmo_rtp_socket *rs)
+{
+	rtp_session_set_symmetric_rtp(rs->sess, 1);
+	rtp_session_set_connected_mode(rs->sess, 1);
+	rs->flags &= ~OSMO_RTP_F_DISABLED;
+
+	if (rs->flags & OSMO_RTP_F_POLL)
+		return 0;
+	else
+		return osmo_rtp_socket_fdreg(rs);
+}
+
 /*! \brief Increment timestamp on a RTP socket without sending any packet
  *  \param[in] rs OsmoRTP socket
  *  \param[in] duration duration in number of RTP clock ticks
@@ -529,6 +584,16 @@ int osmo_rtp_socket_set_pt(struct osmo_rtp_socket *rs, int payload_type)
 	return rc;
 }
 
+/*! \brief Set the DSCP (Differentiated Services Code Point) for outgoing RTP packets
+ *  \param[in] rs OsmoRTP socket
+ *  \param[in] dscp DSCP value
+ *  \returns 0 on success, < 0 otherwise
+ */
+int osmo_rtp_socket_set_dscp(struct osmo_rtp_socket *rs, int dscp)
+{
+	return rtp_session_set_dscp(rs->sess, dscp);
+}
+
 /*! \brief completely close the RTP socket and release all resources
  *  \param[in] rs OsmoRTP socket to be released
  *  \returns 0 on success
